'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var crypto = _interopDefault(require('crypto'));

// Unique ID creation requires a high quality random # generator.  In node.js
// this is pretty straight-forward - we use the crypto API.



var rng = function nodeRNG() {
  return crypto.randomBytes(16);
};

/**
 * Convert array of 16 byte values to UUID string format of the form:
 * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
 */
var byteToHex = [];
for (var i = 0; i < 256; ++i) {
  byteToHex[i] = (i + 0x100).toString(16).substr(1);
}

function bytesToUuid(buf, offset) {
  var i = offset || 0;
  var bth = byteToHex;
  return bth[buf[i++]] + bth[buf[i++]] +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] +
          bth[buf[i++]] + bth[buf[i++]] +
          bth[buf[i++]] + bth[buf[i++]];
}

var bytesToUuid_1 = bytesToUuid;

// **`v1()` - Generate time-based UUID**
//
// Inspired by https://github.com/LiosK/UUID.js
// and http://docs.python.org/library/uuid.html

var _nodeId;
var _clockseq;

// Previous uuid creation time
var _lastMSecs = 0;
var _lastNSecs = 0;

// See https://github.com/broofa/node-uuid for API details
function v1(options, buf, offset) {
  var i = buf && offset || 0;
  var b = buf || [];

  options = options || {};
  var node = options.node || _nodeId;
  var clockseq = options.clockseq !== undefined ? options.clockseq : _clockseq;

  // node and clockseq need to be initialized to random values if they're not
  // specified.  We do this lazily to minimize issues related to insufficient
  // system entropy.  See #189
  if (node == null || clockseq == null) {
    var seedBytes = rng();
    if (node == null) {
      // Per 4.5, create and 48-bit node id, (47 random bits + multicast bit = 1)
      node = _nodeId = [
        seedBytes[0] | 0x01,
        seedBytes[1], seedBytes[2], seedBytes[3], seedBytes[4], seedBytes[5]
      ];
    }
    if (clockseq == null) {
      // Per 4.2.2, randomize (14 bit) clockseq
      clockseq = _clockseq = (seedBytes[6] << 8 | seedBytes[7]) & 0x3fff;
    }
  }

  // UUID timestamps are 100 nano-second units since the Gregorian epoch,
  // (1582-10-15 00:00).  JSNumbers aren't precise enough for this, so
  // time is handled internally as 'msecs' (integer milliseconds) and 'nsecs'
  // (100-nanoseconds offset from msecs) since unix epoch, 1970-01-01 00:00.
  var msecs = options.msecs !== undefined ? options.msecs : new Date().getTime();

  // Per 4.2.1.2, use count of uuid's generated during the current clock
  // cycle to simulate higher resolution clock
  var nsecs = options.nsecs !== undefined ? options.nsecs : _lastNSecs + 1;

  // Time since last uuid creation (in msecs)
  var dt = (msecs - _lastMSecs) + (nsecs - _lastNSecs)/10000;

  // Per 4.2.1.2, Bump clockseq on clock regression
  if (dt < 0 && options.clockseq === undefined) {
    clockseq = clockseq + 1 & 0x3fff;
  }

  // Reset nsecs if clock regresses (new clockseq) or we've moved onto a new
  // time interval
  if ((dt < 0 || msecs > _lastMSecs) && options.nsecs === undefined) {
    nsecs = 0;
  }

  // Per 4.2.1.2 Throw error if too many uuids are requested
  if (nsecs >= 10000) {
    throw new Error('uuid.v1(): Can\'t create more than 10M uuids/sec');
  }

  _lastMSecs = msecs;
  _lastNSecs = nsecs;
  _clockseq = clockseq;

  // Per 4.1.4 - Convert from unix epoch to Gregorian epoch
  msecs += 12219292800000;

  // `time_low`
  var tl = ((msecs & 0xfffffff) * 10000 + nsecs) % 0x100000000;
  b[i++] = tl >>> 24 & 0xff;
  b[i++] = tl >>> 16 & 0xff;
  b[i++] = tl >>> 8 & 0xff;
  b[i++] = tl & 0xff;

  // `time_mid`
  var tmh = (msecs / 0x100000000 * 10000) & 0xfffffff;
  b[i++] = tmh >>> 8 & 0xff;
  b[i++] = tmh & 0xff;

  // `time_high_and_version`
  b[i++] = tmh >>> 24 & 0xf | 0x10; // include version
  b[i++] = tmh >>> 16 & 0xff;

  // `clock_seq_hi_and_reserved` (Per 4.2.2 - include variant)
  b[i++] = clockseq >>> 8 | 0x80;

  // `clock_seq_low`
  b[i++] = clockseq & 0xff;

  // `node`
  for (var n = 0; n < 6; ++n) {
    b[i + n] = node[n];
  }

  return buf ? buf : bytesToUuid_1(b);
}

var v1_1 = v1;

function v4(options, buf, offset) {
  var i = buf && offset || 0;

  if (typeof(options) == 'string') {
    buf = options === 'binary' ? new Array(16) : null;
    options = null;
  }
  options = options || {};

  var rnds = options.random || (options.rng || rng)();

  // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
  rnds[6] = (rnds[6] & 0x0f) | 0x40;
  rnds[8] = (rnds[8] & 0x3f) | 0x80;

  // Copy bytes to buffer, if provided
  if (buf) {
    for (var ii = 0; ii < 16; ++ii) {
      buf[i + ii] = rnds[ii];
    }
  }

  return buf || bytesToUuid_1(rnds);
}

var v4_1 = v4;

var uuid = v4_1;
uuid.v1 = v1_1;
uuid.v4 = v4_1;

var uuid_1 = uuid;

var uuid_2 = uuid_1.v4;

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};









var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};



















var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var List = function (_Component) {
	inherits(List, _Component);

	function List() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, List);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = List.__proto__ || Object.getPrototypeOf(List)).call.apply(_ref, [this].concat(args))), _this), _this.initialState = {
			ids: [],
			byId: {}
		}, _this.state = _this.initialState, _this.getItems = function () {
			return _this.state.ids.map(function (id) {
				return _this.state.byId[id];
			});
		}, _this.normalizeItems = function (items) {
			return items.reduce(function (state, item) {
				// ensure every item has an id property
				var _item = _extends({
					id: uuid_2()
				}, item);

				return {
					ids: [].concat(toConsumableArray(state.ids), [_item.id]),
					byId: _extends({}, state.byId, defineProperty({}, _item.id, _item))
				};
			}, _this.initialState);
		}, _this.add = function () {
			var item = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			// ensure every item has an id property
			var _item = _extends({
				id: uuid_2()
			}, item);

			_this.setState({
				ids: [].concat(toConsumableArray(_this.state.ids), [_item.id]),
				byId: _extends({}, _this.state.byId, defineProperty({}, _item.id, _item))
			});
		}, _this.remove = function (id) {
			// remove item from byId object by key (id)
			var _this$state$byId = _this.state.byId,
			    _ = _this$state$byId[id],
			    rest = objectWithoutProperties(_this$state$byId, [id]);


			_this.setState({
				ids: _this.state.ids.filter(function (_id) {
					return _id !== id;
				}),
				byId: rest
			});
		}, _this.update = function () {
			var item = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			_this.setState({
				byId: _extends({}, _this.state.byId, defineProperty({}, item.id, _extends({}, _this.state.byId[item.id], item)))
			});
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(List, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			var items = this.normalizeItems(this.props.initial);
			this.setState(_extends({}, items));
		}
	}, {
		key: 'componentDidUpdate',
		value: function componentDidUpdate(prevProps) {
			if (prevProps.initial.length === 0 && this.props.initial.length > 0) {
				var items = this.normalizeItems(this.props.initial);
				this.setState(_extends({}, items));
			}
		}
	}, {
		key: 'render',
		value: function render() {
			if (!this.props.render) return null;

			var items = this.getItems();

			return this.props.render({
				items: items,
				add: this.add,
				remove: this.remove,
				update: this.update
			});
		}
	}]);
	return List;
}(React.Component);

List.defaultProps = {
	initial: []
};

var Dropdown = function (_Component) {
	inherits(Dropdown, _Component);

	function Dropdown() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Dropdown);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Dropdown.__proto__ || Object.getPrototypeOf(Dropdown)).call.apply(_ref, [this].concat(args))), _this), _this.initialState = {
			selected: _this.props.initial ? _this.props.initial : _this.props.items[0] || {},
			isOpen: false
		}, _this.state = _this.initialState, _this.select = function (item) {
			return _this.setState({ selected: item });
		}, _this.toggle = function () {
			return _this.setState({ isOpen: !_this.state.isOpen });
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Dropdown, [{
		key: 'render',
		value: function render() {
			var _this2 = this;

			if (!this.props.render) return null;

			var filter = function filter(item) {
				return item.id !== _this2.state.selected.id;
			};

			return React__default.createElement(List, {
				items: this.props.items,
				render: function render(_ref2) {
					var items = _ref2.items;

					return _this2.props.render({
						select: _this2.select,
						toggle: _this2.toggle,
						selected: _this2.state.selected,
						options: _this2.props.items.filter(filter),
						isOpen: _this2.state.isOpen
					});
				}
			});
		}
	}]);
	return Dropdown;
}(React.Component);

Dropdown.defaultProps = {
	items: []
};

var Fetch = function (_Component) {
	inherits(Fetch, _Component);

	function Fetch() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Fetch);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Fetch.__proto__ || Object.getPrototypeOf(Fetch)).call.apply(_ref, [this].concat(args))), _this), _this.initialState = {
			status: Fetch.STATES.MOUNTED,
			response: {}
		}, _this.state = _this.initialState, _this.__isMounted = false, _this.fetch = function () {
			var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			if (_this.props.api) {
				_this.setState({ status: Fetch.STATES.FETCHING }, function () {
					_this.props.api(params).then(function (response) {
						if (_this.__isMounted) {
							_this.setState({
								response: response,
								status: Fetch.STATES.FETCHED
							});
						}
					}).catch(function (err) {
						if (_this.__isMounted) {
							_this.setState({
								status: Fetch.STATES.ERROR
							});
						}
					});
				});
			}
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Fetch, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			this.__isMounted = true;
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.__isMounted = false;
		}
	}, {
		key: 'render',
		value: function render() {
			if (!this.props.render) return null;

			return this.props.render({
				fetch: this.fetch,
				status: this.state.status,
				response: this.state.response
			});
		}
	}]);
	return Fetch;
}(React.Component);

Fetch.STATES = {
	MOUNTED: 'MOUNTED',
	FETCHING: 'FETCHING',
	FETCHED: 'FETCHED',
	ERROR: 'ERRORED'
};

// manages inputValue (controlled and uncontrolled)
// calls a fetch with inputValue

var Input = function (_Component) {
	inherits(Input, _Component);

	function Input() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Input);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Input.__proto__ || Object.getPrototypeOf(Input)).call.apply(_ref, [this].concat(args))), _this), _this.initialState = {
			inputValue: _this.props.value || ''
		}, _this.state = _this.initialState, _this.isControlled = function () {
			return _this.props.value !== undefined;
		}, _this.handleInputChange = function (event) {
			_this.isControlled() ? _this.props.onChange(event) : _this.setState({ inputValue: event.target.value });
		}, _this.handleKeyPress = function (event) {
			if (event.key === 'Enter') {
				if (_this.props.onEnter) {
					var value = _this.isControlled() ? _this.props.value : _this.state.inputValue;

					_this.props.onEnter({ text: value });
				}
			}
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Input, [{
		key: 'render',
		value: function render() {
			var currentValue = this.isControlled() ? this.props.value : this.state.inputValue;

			return React__default.createElement('input', {
				type: 'text',
				onChange: this.handleInputChange,
				onKeyPress: this.handleKeyPress,
				value: currentValue
			});
		}
	}]);
	return Input;
}(React.Component);

var Update = function (_Component) {
    inherits(Update, _Component);

    function Update() {
        var _ref;

        var _temp, _this, _ret;

        classCallCheck(this, Update);

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Update.__proto__ || Object.getPrototypeOf(Update)).call.apply(_ref, [this].concat(args))), _this), _this.componentDidUpdate = function (prevProps) {
            if (_this.props.value !== prevProps.value) {
                _this.props.onUpdate(_this.props.value);
            }
        }, _temp), possibleConstructorReturn(_this, _ret);
    }

    createClass(Update, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.props.onUpdate(this.props.value);
        }
    }, {
        key: 'render',
        value: function render() {
            return this.props.children ? this.props.children : null;
        }
    }]);
    return Update;
}(React.Component);

Update.defaultProps = {
    value: null,
    onUpdate: function onUpdate() {}
};

exports.Dropdown = Dropdown;
exports.Fetch = Fetch;
exports.Input = Input;
exports.List = List;
exports.Update = Update;
//# sourceMappingURL=index.js.map
